@@ -1,46 +1,41 @@
 from __future__ import annotations
 
 import operator
-from typing import TYPE_CHECKING
-from typing import Any
-from typing import Callable
-from typing import Literal
-from typing import Sequence
-from typing import cast
+from functools import partial
+from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar, cast
 
 import ibis
-from ibis import _ as col
 
 from narwhals._compliant import LazyExpr
-from narwhals._expression_parsing import ExprKind
+from narwhals._compliant.window import WindowInputs
 from narwhals._ibis.expr_dt import IbisExprDateTimeNamespace
 from narwhals._ibis.expr_list import IbisExprListNamespace
 from narwhals._ibis.expr_str import IbisExprStringNamespace
 from narwhals._ibis.expr_struct import IbisExprStructNamespace
-from narwhals._ibis.utils import WindowInputs
-from narwhals._ibis.utils import is_floating
-from narwhals._ibis.utils import lit
-from narwhals._ibis.utils import narwhals_to_native_dtype
-from narwhals.utils import Implementation
-from narwhals.utils import not_implemented
+from narwhals._ibis.utils import is_floating, lit, narwhals_to_native_dtype
+from narwhals._utils import Implementation, not_implemented
 
 if TYPE_CHECKING:
+    from collections.abc import Iterator, Sequence
+
     import ibis.expr.types as ir
     from typing_extensions import Self
 
-    from narwhals._compliant.typing import AliasNames
-    from narwhals._compliant.typing import EvalNames
-    from narwhals._compliant.typing import EvalSeries
-    from narwhals._expression_parsing import ExprMetadata
+    from narwhals._compliant.typing import (
+        AliasNames,
+        EvalNames,
+        EvalSeries,
+        WindowFunction,
+    )
+    from narwhals._expression_parsing import ExprKind, ExprMetadata
     from narwhals._ibis.dataframe import IbisLazyFrame
     from narwhals._ibis.namespace import IbisNamespace
-    from narwhals._ibis.typing import ExprT
-    from narwhals._ibis.typing import WindowFunction
-    from narwhals.dtypes import DType
-    from narwhals.typing import RankMethod
-    from narwhals.typing import RollingInterpolationMethod
-    from narwhals.utils import Version
-    from narwhals.utils import _FullContext
+    from narwhals._utils import Version, _FullContext
+    from narwhals.typing import IntoDType, RankMethod, RollingInterpolationMethod
+
+    ExprT = TypeVar("ExprT", bound=ir.Value)
+    IbisWindowFunction = WindowFunction[IbisLazyFrame, ir.Value]
+    IbisWindowInputs = WindowInputs[ir.Value]
 
 
 class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
@@ -49,6 +44,7 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
     def __init__(
         self,
         call: EvalSeries[IbisLazyFrame, ir.Value],
+        window_function: IbisWindowFunction | None = None,
         *,
         evaluate_output_names: EvalNames[IbisLazyFrame],
         alias_output_names: AliasNames | None,
@@ -61,9 +57,20 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
         self._backend_version = backend_version
         self._version = version
         self._metadata: ExprMetadata | None = None
+        self._window_function: IbisWindowFunction | None = window_function
 
-        # This can only be set by `_with_window_function`.
-        self._window_function: WindowFunction | None = None
+    @property
+    def window_function(self) -> IbisWindowFunction:
+        def default_window_func(
+            df: IbisLazyFrame, window_inputs: IbisWindowInputs
+        ) -> list[ir.Value]:
+            assert not window_inputs.order_by  # noqa: S101
+            return [
+                expr.over(ibis.window(group_by=window_inputs.partition_by))
+                for expr in self(df)
+            ]
+
+        return self._window_function or default_window_func
 
     def __call__(self, df: IbisLazyFrame) -> Sequence[ir.Value]:
         return self._call(df)
@@ -78,27 +85,18 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
 
     def _cum_window_func(
         self, *, reverse: bool, func_name: Literal["sum", "max", "min", "count"]
-    ) -> WindowFunction:
-        def func(window_inputs: WindowInputs) -> ir.Value:
-            if reverse:
-                order_by_cols = [
-                    ibis.desc(getattr(col, x), nulls_first=False)
-                    for x in window_inputs.order_by
-                ]
-            else:
-                order_by_cols = [
-                    ibis.asc(getattr(col, x), nulls_first=True)
-                    for x in window_inputs.order_by
-                ]
-
+    ) -> IbisWindowFunction:
+        def func(df: IbisLazyFrame, inputs: IbisWindowInputs) -> Sequence[ir.Value]:
             window = ibis.window(
-                group_by=list(window_inputs.partition_by),
-                order_by=order_by_cols,
+                group_by=list(inputs.partition_by),
+                order_by=self._sort(
+                    *inputs.order_by, descending=reverse, nulls_last=reverse
+                ),
                 preceding=None,  # unbounded
                 following=0,
             )
 
-            return getattr(window_inputs.expr, func_name)().over(window)
+            return [getattr(expr, func_name)().over(window) for expr in self(df)]
 
         return func
 
@@ -110,7 +108,7 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
         window_size: int,
         min_samples: int,
         ddof: int | None = None,
-    ) -> WindowFunction:
+    ) -> IbisWindowFunction:
         supported_funcs = ["sum", "mean", "std", "var"]
 
         if center:
@@ -120,62 +118,58 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
             preceding = window_size - 1
             following = 0
 
-        def func(window_inputs: WindowInputs) -> ir.Value:
-            order_by_cols = [
-                ibis.asc(getattr(col, x), nulls_first=True)
-                for x in window_inputs.order_by
-            ]
+        def func(df: IbisLazyFrame, inputs: IbisWindowInputs) -> Sequence[ir.Value]:
             window = ibis.window(
-                group_by=list(window_inputs.partition_by),
-                order_by=order_by_cols,
+                group_by=list(inputs.partition_by),
+                order_by=self._sort(*inputs.order_by),
                 preceding=preceding,
                 following=following,
             )
 
-            expr: ir.NumericColumn = cast("ir.NumericColumn", window_inputs.expr)
+            def inner_f(expr: ir.NumericColumn) -> ir.Value:
+                if func_name in {"sum", "mean"}:
+                    func_ = getattr(expr, func_name)()
+                elif func_name == "var" and ddof == 0:
+                    func_ = expr.var(how="pop")
+                elif func_name in "var" and ddof == 1:
+                    func_ = expr.var(how="sample")
+                elif func_name == "std" and ddof == 0:
+                    func_ = expr.std(how="pop")
+                elif func_name == "std" and ddof == 1:
+                    func_ = expr.std(how="sample")
+                elif func_name in {"var", "std"}:  # pragma: no cover
+                    msg = f"Only ddof=0 and ddof=1 are currently supported for rolling_{func_name}."
+                    raise ValueError(msg)
+                else:  # pragma: no cover
+                    msg = f"Only the following functions are supported: {supported_funcs}.\nGot: {func_name}."
+                    raise ValueError(msg)
 
-            func_: ir.NumericScalar
+                rolling_calc = func_.over(window)
+                valid_count = expr.count().over(window)
+                return ibis.cases(
+                    (valid_count >= ibis.literal(min_samples), rolling_calc),
+                    else_=ibis.null(),
+                )
 
-            if func_name in {"sum", "mean"}:
-                func_ = getattr(expr, func_name)()
-            elif func_name == "var" and ddof == 0:
-                func_ = expr.var(how="pop")
-            elif func_name in "var" and ddof == 1:
-                func_ = expr.var(how="sample")
-            elif func_name == "std" and ddof == 0:
-                func_ = expr.std(how="pop")
-            elif func_name == "std" and ddof == 1:
-                func_ = expr.std(how="sample")
-            elif func_name in {"var", "std"}:  # pragma: no cover
-                msg = f"Only ddof=0 and ddof=1 are currently supported for rolling_{func_name}."
-                raise ValueError(msg)
-            else:  # pragma: no cover
-                msg = f"Only the following functions are supported: {supported_funcs}.\nGot: {func_name}."
-                raise ValueError(msg)
-
-            rolling_calc = func_.over(window)
-            valid_count = expr.count().over(window)
-            return ibis.cases(
-                (valid_count >= ibis.literal(min_samples), rolling_calc),
-                else_=ibis.null(),
-            )
+            return [inner_f(cast("ir.NumericColumn", expr)) for expr in self(df)]
 
         return func
 
     def broadcast(self, kind: Literal[ExprKind.AGGREGATION, ExprKind.LITERAL]) -> Self:
-        if kind is ExprKind.LITERAL:
-            return self
+        # Ibis does its own broadcasting.
+        return self
 
-        def func(df: IbisLazyFrame) -> Sequence[ir.Value]:
-            return [expr.over() for expr in self(df)]
-
-        return self.__class__(
-            func,
-            evaluate_output_names=self._evaluate_output_names,
-            alias_output_names=self._alias_output_names,
-            backend_version=self._backend_version,
-            version=self._version,
-        )
+    def _sort(
+        self, *cols: ir.Column | str, descending: bool = False, nulls_last: bool = False
+    ) -> Iterator[ir.Column]:
+        mapping = {
+            (False, False): partial(ibis.asc, nulls_first=True),
+            (False, True): partial(ibis.asc, nulls_first=False),
+            (True, False): partial(ibis.desc, nulls_first=True),
+            (True, True): partial(ibis.desc, nulls_first=False),
+        }
+        sort = mapping[(descending, nulls_last)]
+        yield from (cast("ir.Column", sort(col)) for col in cols)
 
     @classmethod
     def from_column_names(
@@ -242,120 +236,118 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
 
     def _with_alias_output_names(self, func: AliasNames | None, /) -> Self:
         return type(self)(
-            call=self._call,
+            self._call,
+            self._window_function,
             evaluate_output_names=self._evaluate_output_names,
             alias_output_names=func,
             backend_version=self._backend_version,
             version=self._version,
         )
 
-    def _with_window_function(self, window_function: WindowFunction) -> Self:
-        result = self.__class__(
+    def _with_window_function(self, window_function: IbisWindowFunction) -> Self:
+        return self.__class__(
             self._call,
+            window_function,
             evaluate_output_names=self._evaluate_output_names,
             alias_output_names=self._alias_output_names,
             backend_version=self._backend_version,
             version=self._version,
         )
-        result._window_function = window_function
-        return result
 
     @classmethod
     def _alias_native(cls, expr: ExprT, name: str, /) -> ExprT:
         return cast("ExprT", expr.name(name))
 
     def __and__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda _input, other: _input & other, other=other)
+        return self._with_callable(lambda expr, other: expr & other, other=other)
 
     def __or__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda _input, other: _input | other, other=other)
+        return self._with_callable(lambda expr, other: expr | other, other=other)
 
     def __add__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda _input, other: _input + other, other=other)
+        return self._with_callable(lambda expr, other: expr + other, other=other)
 
     def __truediv__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda _input, other: _input / other, other=other)
+        return self._with_callable(lambda expr, other: expr / other, other=other)
 
     def __rtruediv__(self, other: IbisExpr) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__rtruediv__(other), other=other
+            lambda expr, other: expr.__rtruediv__(other), other=other
         ).alias("literal")
 
     def __floordiv__(self, other: IbisExpr) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__floordiv__(other), other=other
+            lambda expr, other: expr.__floordiv__(other), other=other
         )
 
     def __rfloordiv__(self, other: IbisExpr) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__rfloordiv__(other), other=other
+            lambda expr, other: expr.__rfloordiv__(other), other=other
         ).alias("literal")
 
     def __mod__(self, other: IbisExpr) -> Self:
-        return self._with_callable(
-            lambda _input, other: _input.__mod__(other), other=other
-        )
+        return self._with_callable(lambda expr, other: expr.__mod__(other), other=other)
 
     def __rmod__(self, other: IbisExpr) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__rmod__(other), other=other
+            lambda expr, other: expr.__rmod__(other), other=other
         ).alias("literal")
 
     def __sub__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda _input, other: _input - other, other=other)
+        return self._with_callable(lambda expr, other: expr - other, other=other)
 
     def __rsub__(self, other: IbisExpr) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__rsub__(other), other=other
+            lambda expr, other: expr.__rsub__(other), other=other
         ).alias("literal")
 
     def __mul__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda _input, other: _input * other, other=other)
+        return self._with_callable(lambda expr, other: expr * other, other=other)
 
     def __pow__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda _input, other: _input**other, other=other)
+        return self._with_callable(lambda expr, other: expr**other, other=other)
 
     def __rpow__(self, other: IbisExpr) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__rpow__(other), other=other
+            lambda expr, other: expr.__rpow__(other), other=other
         ).alias("literal")
 
     def __lt__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda _input, other: _input < other, other=other)
+        return self._with_callable(lambda expr, other: expr < other, other=other)
 
     def __gt__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda _input, other: _input > other, other=other)
+        return self._with_callable(lambda expr, other: expr > other, other=other)
 
     def __le__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda _input, other: _input <= other, other=other)
+        return self._with_callable(lambda expr, other: expr <= other, other=other)
 
     def __ge__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda _input, other: _input >= other, other=other)
+        return self._with_callable(lambda expr, other: expr >= other, other=other)
 
     def __eq__(self, other: IbisExpr) -> Self:  # type: ignore[override]
-        return self._with_callable(lambda _input, other: _input == other, other=other)
+        return self._with_callable(lambda expr, other: expr == other, other=other)
 
     def __ne__(self, other: IbisExpr) -> Self:  # type: ignore[override]
-        return self._with_callable(lambda _input, other: _input != other, other=other)
+        return self._with_callable(lambda expr, other: expr != other, other=other)
 
     def __invert__(self) -> Self:
         invert = cast("Callable[..., ir.Value]", operator.invert)
         return self._with_callable(invert)
 
     def abs(self) -> Self:
-        return self._with_callable(lambda _input: _input.abs())
+        return self._with_callable(lambda expr: expr.abs())
 
     def mean(self) -> Self:
-        return self._with_callable(lambda _input: _input.mean())
+        return self._with_callable(lambda expr: expr.mean())
 
     def median(self) -> Self:
-        return self._with_callable(lambda _input: _input.median())
+        return self._with_callable(lambda expr: expr.median())
 
     def all(self) -> Self:
-        return self._with_callable(lambda _input: _input.all())
+        return self._with_callable(lambda expr: expr.all().fill_null(lit(True)))  # noqa: FBT003
 
     def any(self) -> Self:
-        return self._with_callable(lambda _input: _input.any())
+        return self._with_callable(lambda expr: expr.any().fill_null(lit(False)))  # noqa: FBT003
 
     def quantile(
         self, quantile: float, interpolation: RollingInterpolationMethod
@@ -363,24 +355,24 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
         if interpolation != "linear":
             msg = "Only linear interpolation methods are supported for Ibis quantile."
             raise NotImplementedError(msg)
-        return self._with_callable(lambda _input: _input.quantile(quantile))
+        return self._with_callable(lambda expr: expr.quantile(quantile))
 
     def clip(self, lower_bound: Any, upper_bound: Any) -> Self:
-        def _clip(_input: ir.NumericValue, lower: Any, upper: Any) -> ir.NumericValue:
-            return _input.clip(lower=lower, upper=upper)
+        def _clip(expr: ir.NumericValue, lower: Any, upper: Any) -> ir.NumericValue:
+            return expr.clip(lower=lower, upper=upper)
 
         return self._with_callable(_clip, lower=lower_bound, upper=upper_bound)
 
     def sum(self) -> Self:
-        return self._with_callable(lambda _input: _input.sum())
+        return self._with_callable(lambda expr: expr.sum().fill_null(lit(0)))
 
     def n_unique(self) -> Self:
         return self._with_callable(
-            lambda _input: _input.nunique() + _input.isnull().any().cast("int8")
+            lambda expr: expr.nunique() + expr.isnull().any().cast("int8")
         )
 
     def count(self) -> Self:
-        return self._with_callable(lambda _input: _input.count())
+        return self._with_callable(lambda expr: expr.count())
 
     def len(self) -> Self:
         def func(df: IbisLazyFrame) -> list[ir.IntegerScalar]:
@@ -395,54 +387,45 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
         )
 
     def std(self, ddof: int) -> Self:
-        def _std(_input: ir.NumericColumn, ddof: int) -> ir.Value:
+        def _std(expr: ir.NumericColumn, ddof: int) -> ir.Value:
             if ddof == 0:
-                return _input.std(how="pop")
+                return expr.std(how="pop")
             elif ddof == 1:
-                return _input.std(how="sample")
+                return expr.std(how="sample")
             else:
-                n_samples = _input.count()
-                std_pop = _input.std(how="pop")
+                n_samples = expr.count()
+                std_pop = expr.std(how="pop")
                 ddof_lit = cast("ir.IntegerScalar", ibis.literal(ddof))
                 return std_pop * n_samples.sqrt() / (n_samples - ddof_lit).sqrt()
 
-        return self._with_callable(lambda _input: _std(_input, ddof))
+        return self._with_callable(lambda expr: _std(expr, ddof))
 
     def var(self, ddof: int) -> Self:
-        def _var(_input: ir.NumericColumn, ddof: int) -> ir.Value:
+        def _var(expr: ir.NumericColumn, ddof: int) -> ir.Value:
             if ddof == 0:
-                return _input.var(how="pop")
+                return expr.var(how="pop")
             elif ddof == 1:
-                return _input.var(how="sample")
+                return expr.var(how="sample")
             else:
-                n_samples = _input.count()
-                var_pop = _input.var(how="pop")
+                n_samples = expr.count()
+                var_pop = expr.var(how="pop")
                 ddof_lit = cast("ir.IntegerScalar", ibis.literal(ddof))
                 return var_pop * n_samples / (n_samples - ddof_lit)
 
-        return self._with_callable(lambda _input: _var(_input, ddof))
+        return self._with_callable(lambda expr: _var(expr, ddof))
 
     def max(self) -> Self:
-        return self._with_callable(lambda _input: _input.max())
+        return self._with_callable(lambda expr: expr.max())
 
     def min(self) -> Self:
-        return self._with_callable(lambda _input: _input.min())
+        return self._with_callable(lambda expr: expr.min())
 
     def null_count(self) -> Self:
-        return self._with_callable(lambda _input: _input.isnull().sum())
+        return self._with_callable(lambda expr: expr.isnull().sum())
 
-    def over(self, partition_by: Sequence[str], order_by: Sequence[str] | None) -> Self:
-        if (fn := self._window_function) is not None:
-            assert order_by is not None  # noqa: S101
-
-            def func(df: IbisLazyFrame) -> list[ir.Value]:
-                return [
-                    fn(WindowInputs(expr, partition_by, order_by)) for expr in self(df)
-                ]
-        else:
-
-            def func(df: IbisLazyFrame) -> list[ir.Value]:
-                return [expr.over(group_by=partition_by) for expr in self(df)]
+    def over(self, partition_by: Sequence[str], order_by: Sequence[str]) -> Self:
+        def func(df: IbisLazyFrame) -> Sequence[ir.Value]:
+            return self.window_function(df, WindowInputs(partition_by, order_by))
 
         return self.__class__(
             func,
@@ -453,64 +436,89 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
         )
 
     def is_null(self) -> Self:
-        return self._with_callable(lambda _input: _input.isnull())
+        return self._with_callable(lambda expr: expr.isnull())
 
     def is_nan(self) -> Self:
-        def func(_input: ir.FloatingValue | Any) -> ir.Value:
-            otherwise = _input.isnan() if is_floating(_input.type()) else False
-            return ibis.ifelse(_input.isnull(), None, otherwise)
+        def func(expr: ir.FloatingValue | Any) -> ir.Value:
+            otherwise = expr.isnan() if is_floating(expr.type()) else False
+            return ibis.ifelse(expr.isnull(), None, otherwise)
 
         return self._with_callable(func)
 
     def is_finite(self) -> Self:
-        return self._with_callable(lambda _input: ~(_input.isinf() | _input.isnan()))
+        return self._with_callable(lambda expr: ~(expr.isinf() | expr.isnan()))
 
     def is_in(self, other: Sequence[Any]) -> Self:
-        return self._with_callable(lambda _input: _input.isin(other))
+        return self._with_callable(lambda expr: expr.isin(other))
 
     def round(self, decimals: int) -> Self:
-        return self._with_callable(lambda _input: _input.round(decimals))
+        return self._with_callable(lambda expr: expr.round(decimals))
 
     def shift(self, n: int) -> Self:
-        def _func(window_inputs: WindowInputs) -> ir.Column:
-            expr = cast("ir.Column", window_inputs.expr)
-            return expr.lag(n)
+        def _func(df: IbisLazyFrame, inputs: IbisWindowInputs) -> Sequence[ir.Value]:
+            return [
+                expr.lag(n).over(  # type: ignore[attr-defined, unused-ignore]
+                    ibis.window(
+                        group_by=inputs.partition_by,
+                        order_by=self._sort(*inputs.order_by),
+                    )
+                )
+                for expr in self(df)
+            ]
 
         return self._with_window_function(_func)
 
     def is_first_distinct(self) -> Self:
-        def func(window_inputs: WindowInputs) -> ir.BooleanValue:
-            order_by_cols = [
-                ibis.asc(getattr(col, x), nulls_first=True)
-                for x in window_inputs.order_by
-            ]
-            window = ibis.window(
-                group_by=[*window_inputs.partition_by, window_inputs.expr],
-                order_by=order_by_cols,
-            )
+        def func(
+            df: IbisLazyFrame, inputs: IbisWindowInputs
+        ) -> Sequence[ir.BooleanValue]:
             # ibis row_number starts at 0, so need to compare with 0 instead of the usual `1`
-            return ibis.row_number().over(window) == lit(0)
+            return [
+                ibis.row_number().over(
+                    ibis.window(
+                        group_by=[*inputs.partition_by, expr],
+                        order_by=self._sort(*inputs.order_by),
+                    )
+                )
+                == lit(0)
+                for expr in self(df)
+            ]
 
         return self._with_window_function(func)
 
     def is_last_distinct(self) -> Self:
-        def func(window_inputs: WindowInputs) -> ir.Value:
-            order_by_cols = [ibis.desc(getattr(col, x)) for x in window_inputs.order_by]
-            window = ibis.window(
-                group_by=[*window_inputs.partition_by, window_inputs.expr],
-                order_by=order_by_cols,
-            )
+        def func(
+            df: IbisLazyFrame, inputs: IbisWindowInputs
+        ) -> Sequence[ir.BooleanValue]:
             # ibis row_number starts at 0, so need to compare with 0 instead of the usual `1`
-            return ibis.row_number().over(window) == lit(0)
+            return [
+                ibis.row_number().over(
+                    ibis.window(
+                        group_by=[*inputs.partition_by, expr],
+                        order_by=self._sort(
+                            *inputs.order_by, descending=True, nulls_last=True
+                        ),
+                    )
+                )
+                == lit(0)
+                for expr in self(df)
+            ]
 
         return self._with_window_function(func)
 
     def diff(self) -> Self:
-        def _func(window_inputs: WindowInputs) -> ir.NumericValue:
-            expr = cast("ir.NumericColumn", window_inputs.expr)
-            return expr - cast(
-                "ir.NumericColumn", expr.lag().over(ibis.window(following=0))
-            )
+        def _func(df: IbisLazyFrame, inputs: IbisWindowInputs) -> Sequence[ir.Value]:
+            return [
+                expr
+                - expr.lag().over(  # type: ignore[attr-defined, unused-ignore]
+                    ibis.window(
+                        following=0,
+                        group_by=inputs.partition_by,
+                        order_by=self._sort(*inputs.order_by),
+                    )
+                )
+                for expr in self(df)
+            ]
 
         return self._with_window_function(_func)
 
@@ -590,32 +598,27 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
             msg = "`limit` is not supported for the Ibis backend"  # pragma: no cover
             raise NotImplementedError(msg)
 
-        def _fill_null(_input: ir.Value, value: ir.Scalar) -> ir.Value:
-            return _input.fill_null(value)
+        def _fill_null(expr: ir.Value, value: ir.Scalar) -> ir.Value:
+            return expr.fill_null(value)
 
         return self._with_callable(_fill_null, value=value)
 
-    def cast(self, dtype: DType | type[DType]) -> Self:
-        def _func(_input: ir.Column) -> ir.Value:
+    def cast(self, dtype: IntoDType) -> Self:
+        def _func(expr: ir.Column) -> ir.Value:
             native_dtype = narwhals_to_native_dtype(dtype, self._version)
             # ibis `cast` overloads do not include DataType, only literals
-            return _input.cast(native_dtype)  # type: ignore[unused-ignore]
+            return expr.cast(native_dtype)  # type: ignore[unused-ignore]
 
         return self._with_callable(_func)
 
     def is_unique(self) -> Self:
         return self._with_callable(
-            lambda _input: _input.isnull().count().over(ibis.window(group_by=(_input)))
-            == 1
+            lambda expr: expr.isnull().count().over(ibis.window(group_by=(expr))) == 1
         )
 
     def rank(self, method: RankMethod, *, descending: bool) -> Self:
-        def _rank(_input: ir.Column) -> ir.Column:
-            order_by: ir.Column = (
-                cast("ir.Column", _input.desc())
-                if descending
-                else cast("ir.Column", _input.asc())
-            )
+        def _rank(expr: ir.Column) -> ir.Column:
+            order_by = next(self._sort(expr, descending=descending, nulls_last=True))
             window = ibis.window(order_by=order_by)
 
             if method == "dense":
@@ -630,33 +633,45 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
 
             # For "max" and "average", adjust using the count of rows in the partition.
             if method == "max":
-                # Define a window partitioned by _input (i.e. each distinct value)
-                partition = ibis.window(group_by=[_input])
-                cnt = cast("ir.IntegerValue", _input.count().over(partition))
+                # Define a window partitioned by expr (i.e. each distinct value)
+                partition = ibis.window(group_by=[expr])
+                cnt = cast("ir.IntegerValue", expr.count().over(partition))
                 rank_ = rank_ + cnt - cast("ir.IntegerValue", lit(1))
             elif method == "average":
-                partition = ibis.window(group_by=[_input])
-                cnt = cast("ir.IntegerValue", _input.count().over(partition))
+                partition = ibis.window(group_by=[expr])
+                cnt = cast("ir.IntegerValue", expr.count().over(partition))
                 avg = cast(
                     "ir.NumericValue", (cnt - cast("ir.IntegerScalar", lit(1))) / lit(2.0)
                 )
                 rank_ = rank_ + avg
 
-            return cast("ir.Column", ibis.cases((_input.notnull(), rank_)))
+            return cast("ir.Column", ibis.cases((expr.notnull(), rank_)))
 
         return self._with_callable(_rank)
 
     def log(self, base: float) -> Self:
-        def _log(_input: ir.NumericColumn) -> ir.Value:
-            otherwise = _input.log(cast("ir.NumericValue", lit(base)))
+        def _log(expr: ir.NumericColumn) -> ir.Value:
+            otherwise = expr.log(cast("ir.NumericValue", lit(base)))
             return ibis.cases(
-                (_input < lit(0), lit(float("nan"))),
-                (_input == lit(0), lit(float("-inf"))),
+                (expr < lit(0), lit(float("nan"))),
+                (expr == lit(0), lit(float("-inf"))),
                 else_=otherwise,
             )
 
         return self._with_callable(_log)
 
+    def exp(self) -> Self:
+        def _exp(expr: ir.NumericColumn) -> ir.Value:
+            return expr.exp()
+
+        return self._with_callable(_exp)
+
+    def sqrt(self) -> Self:
+        def _sqrt(expr: ir.NumericColumn) -> ir.Value:
+            return ibis.cases((expr < lit(0), lit(float("nan"))), else_=expr.sqrt())
+
+        return self._with_callable(_sqrt)
+
     @property
     def str(self) -> IbisExprStringNamespace:
         return IbisExprStringNamespace(self)
@@ -679,4 +694,5 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
 
     # NOTE: https://github.com/ibis-project/ibis/issues/11176
     skew = not_implemented()
+    kurtosis = not_implemented()
     unique = not_implemented()
